@@ -19,10 +19,15 @@
  * Author: Ryan Lortie <desrt@desrt.ca>
  */
 
+#include "config.h"
+
+#include <locale.h>
 #include <string.h>
 #include <unistd.h>
 #include <stdio.h>
 
+#include <gi18n.h>
+
 #include "gvdb/gvdb-builder.h"
 
 typedef struct
@@ -48,6 +53,8 @@ typedef struct
   GVariantType *type;
 } ParseState;
 
+static gboolean allow_any_name = FALSE;
+
 static gboolean
 is_valid_keyname (const gchar  *key,
                   GError      **error)
@@ -61,6 +68,9 @@ is_valid_keyname (const gchar  *key,
       return FALSE;
     }
 
+  if (allow_any_name)
+    return TRUE;
+
   if (!g_ascii_islower (key[0]))
     {
       g_set_error (error, G_MARKUP_ERROR, G_MARKUP_ERROR_INVALID_CONTENT,
@@ -131,7 +141,7 @@ start_element (GMarkupParseContext  *context,
 #define OPTIONAL   G_MARKUP_COLLECT_OPTIONAL
 #define STRDUP     G_MARKUP_COLLECT_STRDUP
 #define STRING     G_MARKUP_COLLECT_STRING
-#define NO_ARGS()  COLLECT (G_MARKUP_COLLECT_INVALID, NULL)
+#define NO_ATTRS()  COLLECT (G_MARKUP_COLLECT_INVALID, NULL)
 
   if (container == NULL)
     {
@@ -302,12 +312,12 @@ start_element (GMarkupParseContext  *context,
                strcmp (element_name, "description") == 0)
         {
           state->string = g_string_new (NULL);
-          NO_ARGS ();
+          NO_ATTRS ();
           return;
         }
       else if (strcmp (element_name, "range") == 0)
         {
-          NO_ARGS ();
+          NO_ATTRS ();
           return;
         }
     }
@@ -325,12 +335,12 @@ start_element (GMarkupParseContext  *context,
         }
       else if (strcmp (element_name, "min") == 0)
         {
-          NO_ARGS ();
+          NO_ATTRS ();
           return;
         }
       else if (strcmp (element_name, "max") == 0)
         {
-          NO_ARGS ();
+          NO_ATTRS ();
           return;
         }
     }
@@ -385,7 +395,9 @@ end_element (GMarkupParseContext  *context,
         }
 
       g_string_free (state->string, TRUE);
+      state->string = NULL;
       g_free (state->context);
+      state->context = NULL;
     }
 
   else if (strcmp (element_name, "key") == 0)
@@ -405,7 +417,10 @@ end_element (GMarkupParseContext  *context,
 
   else if (strcmp (element_name, "summary") == 0 ||
            strcmp (element_name, "description") == 0)
-    g_string_free (state->string, TRUE);
+    {
+      g_string_free (state->string, TRUE);
+      state->string = NULL;
+    }
 }
 
 static void
@@ -476,77 +491,96 @@ int
 main (int argc, char **argv)
 {
   gboolean byteswap = G_BYTE_ORDER != G_LITTLE_ENDIAN;
-  GError *error = NULL;
+  GError *error;
   GHashTable *table;
   GDir *dir;
   const gchar *file;
-  GPtrArray *files;
   gchar *srcdir;
   gchar *targetdir = NULL;
   gchar *target;
+  gboolean dry_run = FALSE;
+  gchar **schema_files = NULL;
   GOptionContext *context;
   GOptionEntry entries[] = {
-    { "targetdir", 0, 0, G_OPTION_ARG_FILENAME, &targetdir, "where to store the gschemas.compiled file", "DIRECTORY" },
+    { "targetdir", 0, 0, G_OPTION_ARG_FILENAME, &targetdir, N_("where to store the gschemas.compiled file"), N_("DIRECTORY") },
+    { "dry-run", 0, 0, G_OPTION_ARG_NONE, &dry_run, N_("Do not write the gschema.compiled file"), NULL },
+    { "allow-any-name", 0, 0, G_OPTION_ARG_NONE, &allow_any_name, N_("Do not enforce key name restrictions") },
+
+    /* These options are only for use in the gschema-compile tests */
+    { "schema-files", 0, G_OPTION_FLAG_HIDDEN, G_OPTION_ARG_FILENAME_ARRAY, &schema_files, NULL, NULL },
     { NULL }
   };
 
-  context = g_option_context_new ("DIRECTORY");
+  setlocale (LC_ALL, "");
 
+  context = g_option_context_new (N_("DIRECTORY"));
+  g_option_context_set_translation_domain (context, GETTEXT_PACKAGE);
   g_option_context_set_summary (context,
-    "Compile all GSettings schema files into a schema cache.\n"
-    "Schema files are required to have the extension .gschema.xml,\n"
-    "and the cache file is called gschemas.compiled.");
-
-  g_option_context_add_main_entries (context, entries, NULL);
+    N_("Compile all GSettings schema files into a schema cache.\n"
+       "Schema files are required to have the extension .gschema.xml,\n"
+       "and the cache file is called gschemas.compiled."));
+  g_option_context_add_main_entries (context, entries, GETTEXT_PACKAGE);
 
+  error = NULL;
   if (!g_option_context_parse (context, &argc, &argv, &error))
     {
       fprintf (stderr, "%s", error->message);
       return 1;
     }
 
-  if (argc != 2)
+  g_option_context_free (context);
+
+  if (!schema_files && argc != 2)
     {
-      fprintf (stderr, "you should give exactly one directory name\n");
+      fprintf (stderr, _("You should give exactly one directory name\n"));
       return 1;
     }
 
   srcdir = argv[1];
-  if (targetdir)
-    target = g_build_filename (targetdir, "gschemas.compiled", NULL);
-  else
-    target = "gschemas.compiled";
 
-  dir = g_dir_open (srcdir, 0, &error);
-  if (dir == NULL)
+  if (targetdir == NULL)
+    targetdir = srcdir;
+
+  target = g_build_filename (targetdir, "gschemas.compiled", NULL);
+
+  if (!schema_files)
+    {
+      GPtrArray *files;
+
+      files = g_ptr_array_new ();
+
+      dir = g_dir_open (srcdir, 0, &error);
+      if (dir == NULL)
+        {
+          fprintf (stderr, "%s\n", error->message);
+          return 1;
+        }
+
+      while ((file = g_dir_read_name (dir)) != NULL)
+        {
+          if (g_str_has_suffix (file, ".gschema.xml"))
+            g_ptr_array_add (files, g_build_filename (srcdir, file, NULL));
+        }
+
+      if (files->len == 0)
+        {
+          fprintf (stderr, _("No schema files found\n"));
+          return 1;
+        }
+      g_ptr_array_add (files, NULL);
+
+      schema_files = (char **) g_ptr_array_free (files, FALSE);
+    }
+
+
+  if (!(table = parse_gschema_files (schema_files, byteswap, &error)) ||
+      (!dry_run && !gvdb_table_write_contents (table, target, byteswap, &error)))
     {
       fprintf (stderr, "%s\n", error->message);
       return 1;
     }
 
-  files = g_ptr_array_new ();
-  while ((file = g_dir_read_name (dir)) != NULL)
-    {
-      if (g_str_has_suffix (file, ".gschema.xml"))
-	{
-	  g_ptr_array_add (files, g_build_filename (srcdir, file, NULL));
-	}
-    }
-
-  if (files->len == 0)
-    {
-      fprintf (stderr, "no schema files found\n");
-      return 1;
-    }
-
-  g_ptr_array_add (files, NULL);
-
-  if (!(table = parse_gschema_files ((gchar **) files->pdata, byteswap, &error)) ||
-      !gvdb_table_write_contents (table, target, byteswap, &error))
-    {
-      fprintf (stderr, "%s\n", error->message);
-      return 1;
-    }
+  g_free (target);
 
   return 0;
 }
